; THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR
; REPRESENTATIONS, EITHER EXPRESS, IMPLIED OR STATUTORY, 
; INCLUDING ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS 
; FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY OR 
; COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE. 
; TI DISCLAIMS ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET 
; POSSESSION, AND NON-INFRINGEMENT OF ANY THIRD PARTY 
; INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR 
; YOUR USE OF THE PROGRAM.
;
; IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL, 
; CONSEQUENTIAL OR INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY 
; THEORY OF LIABILITY AND WHETHER OR NOT TI HAS BEEN ADVISED 
; OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY OUT 
; OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM. 
; EXCLUDED DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF 
; REMOVAL OR REINSTALLATION, COMPUTER TIME, LABOR COSTS, LOSS 
; OF GOODWILL, LOSS OF PROFITS, LOSS OF SAVINGS, OR LOSS OF 
; USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S 
; AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF 
; YOUR USE OF THE PROGRAM EXCEED FIVE HUNDRED DOLLARS 
; (U.S.$500).
;
; Unless otherwise stated, the Program written and copyrighted 
; by Texas Instruments is distributed as "freeware".  You may, 
; only under TI's copyright in the Program, use and modify the 
; Program without any charge or restriction.  You may 
; distribute to third parties, provided that you transfer a 
; copy of this license to the third party and the third party 
; agrees to these terms by its first use of the Program. You 
; must reproduce the copyright notice and any other legend of 
; ownership on each copy or partial copy, of the Program.
;
; You acknowledge and agree that the Program contains 
; copyrighted material, trade secrets and other TI proprietary 
; information and is protected by copyright laws, 
; international copyright treaties, and trade secret laws, as 
; well as other intellectual property laws.  To protect TI's 
; rights in the Program, you agree not to decompile, reverse 
; engineer, disassemble or otherwise translate any object code 
; versions of the Program to a human-readable form.  You agree 
; that in no event will you alter, remove or destroy any 
; copyright notice included in the Program.  TI reserves all 
; rights not specifically granted under this license. Except 
; as specifically provided herein, nothing in this agreement 
; shall be construed as conferring by implication, estoppel, 
; or otherwise, upon you, any license or other right under any 
; TI patents, copyrights or trade secrets.
;
; You may not use the Program in non-TI devices.

#include <io.h>
;*******************************************************************************
;  "41xThermostat"
;  This program implements a simple thermostat using a MSP430F41x MCU.  The 
;  MSP430 periodically reads the value of a thermistor, displays the reading on
;  an LCD and compares the reading to a setpoint. If the reading is equal to or
;  greater than the setpoint then the LCD flashes and an output goes high.
;  The user is able to change the setpoint, that is also displayed on the LCD 
;  when it is being set. Setpoints are automatically stored in flash memory in
;  case the circuit is powered off.  A real time clock is also implemented and
;  can be displayed on the LCD. The software automatically switches the display
;  from the time to the temperature reading after a preset amount of time.
;
;
;                                              4.5 digit, 3 mux LCD
;                 /|\       MSP430x41x MCU     # VIM-503-DP-FC-S-LV
;                  |        ---------------       --------------
;         +-o\-+(100k)-----| P1.3     COM2 |-----|15 COM3       |
;         |                |'SET'     COM1 |-----|14 COM2       | 
;         |       /|\      |          COM0 |-----|13 COM1       |
;         |        |       |          SEG0 |-----|10            |
;         +-o\-+(100k)-----| P1.2     SEG1 |-----|12            |
;         |                |'MODE'    SEG2 |-----|11            |
;        \|/               |          SEG3 |-----|7             |  
;                +-(10K)---| P1.5     SEG4 |-----|9             |
;                |         |          SEG5 |-----|8             |
;                +-(Sense)-| P1.4     SEG6 |-----|4             |
;                |         |          SEG7 |-----|6             |
;                +---------| P1.6     SEG8 |-----|5             | 
;                |         | CA0      SEG9 |-----|1             |
;               --- 0.1uf  |          SEG10|-----|3             |
;               ---	   |          SEG11|-----|2             |
;                |	   |               |      --------------
;                |	   |               | 
;                +---------|Gnd        P2.5|-----> output high when temperature
;               \|/         ---------------        at or above setpoint
;
;
;  Brian Merritt
;  Texas Instruments Inc.
;  October 2001
;  Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;
;--------CPU register used------------------------------------------------------
#define       Index   R5
			                       ;R5 used to indicate mode =
			                       ;Thermostat, Setpoint, Time or 
			                       ;Seconds
#define       Pointer R6
                                               ;R6 used as info memory pointer
;;;;;;;       R12, R13, R14, R15               ;used as temporary registers in
                                               ;various routines
                                               			                       
;--------Bit definitions used---------------------------------------------------
              .equ    Ref,0x20                 ;reference resisor on P1.5
              .equ    Sensor,0x10              ;thermistor on P1.4
              .equ    SetSW,0x08               ;SET switch on P1.3
              .equ    ModeSW,0x04              ;MODE switch on P1.2

;--------RAM bytes used for variables-------------------------------------------
              .equ    SEC,0x200                 ;byte for counting seconds
              .equ    MIN,0x201                 ;byte for counting minutes
              .equ    HOUR,0x202                ;byte for counting hours
              .equ    FLAG,0x203                ;byte for flag conditions
              .equ    TestFreq,0x204            ;counter to only test temperature 
                                                ;once per set number of seconds
                                                ;to further conserve batteries
              .equ    SetPnt,0x206              ;temperature setpoint 
              .equ    AutoSW,0x207              ;counts seconds in time mode                                               

;--------RAM words used for variables-------------------------------------------                                               
              .equ    LCDTEMP,0x216             ;temporary storage in LCD routine
              .equ    TEMP,0x218                ;temp. storage in temp routine
              .equ    RESULT,0x21A              ;temp. storage in temp routine 
              .equ    IRBIT,0x21C               ;used for bit test
              .equ    IROP1,0x21E               ;first operand
              .equ    IROP2L,0x220              ;second operand low word
              .equ    IROP2H,0x222              ;second operand high word
              .equ    IRACCL,0x224              ;result low word
              .equ    IRACCH,0x226              ;result high word

;-------------------------------------------------------------------------------
;	 Program RESET
;-------------------------------------------------------------------------------
              .section  .text
RESET:        mov    #__stack,R1               ;initialize stackpointer          
              call   #Setup                    ;configure MSP430 for application

Mainloop:     bis    #LPM3,R2                  ;Set SR bits for LPM3
Loop:         br     Mode(Index)               ;branch to mode 

              .align    1                      ;align on even address
Mode:         .word  Thermostat                ;thermostat mode
              .word  Setpoint                  ;input setpoint mode
              .word  Time                      ;display/set real time clock mode
              .word  Second                    ;diplay seconds mode
              
;-------------------------------------------------------------------------------
;	 Display time in hours and minutes
;-------------------------------------------------------------------------------
Time:         dec.b  AutoSW                    ;decrement autoswitch counter
              jnz    TimeDisplay               ;jump if AutoSW > 0
              clr    Index                     ;clear index to 0 = thermostat
              call   #ClearLCD                 ;call routine to clear LCD
              mov.b  #10,AutoSW                ;reload counter for next time              
              jmp    Mainloop                  ;loop back in thermostat mode
TimeDisplay:  call   #Dis_Clock                ;call routine to display time
              xor.b  #0x40,&LCDM1+1            ;blink "." between min and hour
              jmp    Mainloop
;
;-------------------------------------------------------------------------------
;        Display seconds
;------------------------------------------------------------------------------- 
Second:       mov.b  SEC,R14                   ;RAM contents passed via R14 
              call   #ClearLCD                 ;call routine to clear LCD
              mov.b  #LCDM1,R15                ;R15 points to first LCD location
              call   #OutLCD                   ;call routine to display seconds
              jmp    Mainloop

;------------------------------------------------------------------------------- 
;	 Measure, calc' and display temperature, decide if above/below setpoint
;------------------------------------------------------------------------------- 
Thermostat:   cmp.b  #4,TestFreq               ;TestFreq counter = 4?
              jz     MeasTemp                  ;yes, jump to measure temperature
              mov    R12,R14                   ;copy last temp value so R14 can
                                               ;pass value to display routines
              call   #Display1                 ;call routine to display last 
                                               ;reading                                               
              dec.b  TestFreq                  ;decrement counter
              jnz    Mainloop                  ;if not zero then loop                                               
              mov.b  #4,TestFreq               ;reload counter
              jmp    Mainloop                  ;loop

MeasTemp:     call   #Measure                  ;call routines to measure temp
              dec.b  TestFreq                  ;decrement counter
              jmp    Mainloop                  ;loop

;------------------------------------------------------------------------------- 
;        Set temperature value at which temperature display flashes
;------------------------------------------------------------------------------- 
Setpoint:     mov    SetPnt,R14                ;RAM contents passed via R14 
              call   #Display2                 ;call routine to place "SP" on 
                                               ;the LCD then the 2 digit value
              jmp    Mainloop
              
;------------------------------------------------------------------------------- 
;        Measure: Reference, Sensor and Branch to Calculate Subroutine
;------------------------------------------------------------------------------- 
Measure:      mov    #Ref,R14                  ;R14 = Ref pin 
              bic.b  #Ref,&P1DIR               ;change Ref to input, was holding
                                               ;other inputs low - uses less 
                                               ;power then to leave floating
              call   #Charge                   ;charge cap. & measure reference
              mov    R14,TEMP                  ;save Ref discharge time
              mov    #Sensor,R14               ;R14 = Sensor pin
              call   #Charge                   ;charge cap. and measure sensor
              mov    R14,RESULT                ;save Sensor discharge time
              bis.b  #Ref,&P1DIR               ;change Ref to output to hold
                                               ;other inputs low - uses less 
                                               ;power then to leave floating
              br     #Calculate                ;do math, display, then return to
                                               ;Mainloop

;--------Capacitor is charged up with reference resistor------------------------ 
Charge:       mov     #TASSEL1+TACLR+MC1,&TACTL ;SMCLK, clear, continous
              bis.b   #Ref,&P1OUT              ;ref Set
              bis.b   #Ref,&P1DIR              ;ref output
              mov     #5000,&CCR1              ;CCR1 ~ TAR+5ms (5tu)
              mov     #CCIE,&CCTL1             ;compare mode,interrupt
              bis     #LPM0,R2                 ;wait for CCR1 interrupt
              bic.b   #Ref,&P1DIR              ;ref = HiZ, charge complete
              bic.b   #Ref,&P1OUT              ;ref = reset

;--------Measure discharge time ------------------------------------------------
Discharge:    mov.b  #CARSEL+CAREF0+CAON,&CACTL1 ;-Comp = 0.25*Vcc, comp on
              mov.w  #CM1+CCIS0+CAP+CCIE,&CCTL1  ;neg, CCIB,cap,interrupt
              push   &TAR                      ;top of stack = TAR at start of
                                               ;conversion
              bis.b  R14,&P1DIR                ;temp = sensor or ref 
              bis    #LPM0,R2                  ;wait for CCR1 interrupt				 
              mov    &CCR1,R14                 ;R14 = TAR (CCR1) at EOC
              sub    @R1+,R14                  ;R14 = discharge time
              bic.b  #Sensor+Ref,&P1DIR        ;disable sensor or ref 
              clr.b  &CACTL1                   ;disable comparator
              clr    &CCTL1                    ;disable CCTL1
              clr    &TACTL                    ;disable timer_A
              ret                             

;-------------------------------------------------------------------------------
;	 Calculate temperature and fall through to display routines
;-------------------------------------------------------------------------------
Calculate:    mov    RESULT,IROP1
              mov    #10000,IROP2L

;-------------------------------------------------------------------------------
;	 Unsigned multiply routine  IROP1 x IROP2L -> IRACM/IRACL
;	 variables used IROP1, IROP2L, IROP2M, IRACL, IRACM, IRBT
;-------------------------------------------------------------------------------
MPYU:         clr    IRACCL                    ;0 -> LSBs of result
              clr    IRACCH                    ;0 -> MSBs of resut
MACU:         clr    IROP2H                    ;MSBs multiplier
              mov    #1,IRBIT                  ;bit test register
MPY2:         bit    IRBIT,IROP1               ;test actual bit
              jz     MPY1                      ;if 0 do nothing
              add    IROP2L,IRACCL             ;if 1 add multiplier to result
              addc   IROP2H,IRACCH
MPY1:         rla    IROP2L                    ;multiplier x 2
              rlc    IROP2H; 
              rla    IRBIT                     ;next bit to test
              jnc    MPY2                      ;finished if bit in carry
              mov    TEMP,IROP1
              mov    IRACCL,IROP2L			  
              mov    IRACCH,IROP2H

;-------------------------------------------------------------------------------
;        Unsigned division subroutine 32-BIT BY 16-BIT
;        IROP2M|IROP2L : IROP1 -> IRACL	  remainder in IROP2M
;        return: carry = 0 = OK	  carry = 1 = quotient > 16 bits
;        variables used IROP1, IROP2L, IRACL, IRBT, IROP2M
;-------------------------------------------------------------------------------
DIVIDE:       clr    IRACCL                    ;clear accumulator low
              mov    #17,IRBIT                 ;initalize loop counter
DIV1:         cmp    IROP1,IROP2H
              jlo    DIV2
              sub    IROP1,IROP2H
DIV2:         rlc    IRACCL
              jc     RES2F                     ;error: result > 16 bits
              dec    IRBIT                     ;decrement loop counter
              jz     RES2F                     ;is 0: terminate w/o error
              rla    IROP2L
              rlc    IROP2H
              jnc    DIV1
              sub    IROP1,IROP2H
              setc
              jmp    DIV2

;-------------------------------------------------------------------------------
;        Convert sensor resistance value to degrees F for display.
;        table search done directly with BCD subtraction.
;        IRACCL has sensor resistance value
;-------------------------------------------------------------------------------
RES2F:        clr    R13                       ;clear resistance table pointer  
              mov    #99h,R14                  ;R14 used as temperature counter
CMPR:         cmp    RES_TAB(R13),IRACCL       ;compare table value to sensor
              jl     Display1                  ;jump if sensor < table
              incd   R13                       ;double increment (word) pointer
              dadd.b #0x99,R14                 ;subtract 1 from counter
              mov    R14,R12                   ;save in dedicated register in
                                               ;mode is changed temp is not lost
              jmp    CMPR
										
;-------------------------------------------------------------------------------
;        Displays "F", degree sign "o" and temperature on LCD. Toggle LCD on or
;        off with each pass if temp at or above setpoint. R14 passes temp in BCD
;-------------------------------------------------------------------------------
Display1:     mov.b  #0x33,&LCDM1              ;send "F" to display
              xor.b  #0x10,&LCDM1+1            ;blink degree sign (1 of 2 bytes)
              xor.b  #0x13,&LCDM1+2            ;blink degree sign (2 of 2 bytes)              
              mov.b  #LCDM1+3,R15              ;R15 points to first LCD location
CheckTemp:    cmp.b  SetPnt,R14                ;BCD value in R14 equal to or 
                                               ;greater than setpoint value?
              jc     ToggleLCD                 ;yes, jump to toggle LCD
              clr.b  &P2OUT                    ;ensure P2.5 is low
              bis.b  #LCDSON,&LCDCTL           ;ensure LCD display is on              
              jmp    OutLCD                    ;jump to display temp in R14
              
ToggleLCD:    mov.b  #0x20,&P2OUT              ;P2.5 high to indicate temperature
                                               ;at or above setpoint value 
              xor.b  #LCDSON,&LCDCTL           ;toggle LCD on/off on each pass
              jmp    OutLCD                    ;jump to display temp in R14

;-------------------------------------------------------------------------------
;        Displays "SP" followed by 2 digit setpoint
;-------------------------------------------------------------------------------
Display2:     mov.b  #0x33,&LCDM1+3            ;set segments to display S & P
              mov.b  #0x11,&LCDM1+4            ;on LCD. uses 3 LCD memory locals
              mov.b  #0x27,&LCDM1+5            ;due to 3 mux mode
              mov.b  #LCDM1,R15                ;R15 points to first LCD location
              jmp    OutLCD                    ;jump to display setpoint in R14

;-------------------------------------------------------------------------------
;        Display clock values counted MIN, HOUR RAM Bytes. CPU Registers R15 and
;        R14 used temporarily.
;-------------------------------------------------------------------------------
Dis_Clock:    mov.b  #LCDM1,R15;               ;R15 points to first LCD location
              mov.b  MIN,R14                   ;minute BCD value to R14
              call   #OutLCD                   ;call routine to display minutes
              inc    R15                       ;increment to next LCD character
              mov.b  HOUR,R14                  ;hour BCD value to R14
                                               ;fall through to display hours
                                               
;-------------------------------------------------------------------------------
;        Display 2 digits on 3 mux rate LCD. CPU Registers R15, R14 and R13
;        used temporarily.
;-------------------------------------------------------------------------------
OutLCD:       mov.b  R14,R13                   ;copy BCD value in R14 to R13
              rra.b  R13                       ;right shift R13 4 times to 
              rra.b  R13                       ;expose high nibble
              rra.b  R13                       ;
              rra.b  R13                       ;
              and    #0x00F,R14                ;only low nibble now in R14
              add    R14,R14                   ;double R14 since is word data
			                       ;where addresses = 2 bytes apart 
              and    #0x00F,R13                ;only high nibble now in R13

              add    R13,R13                   ;double R13 since is word data
                                               ;where addresses = 2 bytes apart		 
              mov    LCD_Tab(R14),LCDTEMP      ;16 bit segment info into temp                                    
                                               ;LCDTEMP = 0000 0bch 0agd 0yfe
                         
              mov.b  LCDTEMP,0(R15)            ;segments 'dga_ef' of low digit
              swpb   LCDTEMP                   ;swap bytes in temp
              bic.b  #0x37,1(R15)              ;clear all EXCEPT decimal 
                                               ;point, so it can blink
              bis.b  LCDTEMP,1(R15)            ;low byte of temp to LCD, writes
                                               ;segments 'bch' of low digit
                                                           
              mov    LCD_Tab(R13),LCDTEMP      ;16bit segment info into temp                                    
                                               ;LCDTEMP = 0000 0bch 0agd 0yfe
              rla    LCDTEMP                   ;LCDTEMP = 0000 bch0 agd0 yfe0
              rla    LCDTEMP                   ;LCDTEMP = 000b ch0a gd0y fe00
              rla    LCDTEMP                   ;LCDTEMP = 00bc h0ag d0yf e000
              rla    LCDTEMP                   ;LCDTEMP = 0bch 0agd 0yfe 0000
              bis.b  LCDTEMP,1(R15)            ;segments 'yfe' of high digit
              swpb   LCDTEMP                   ;LCDTEMP = 0yfe 0000 0bch 0agd 
              mov.b  LCDTEMP,2(R15)            ;segments 'bchagd' of high digit
              incd.b R15                       ;double inc R15 = LCD mem pointer
              ret
                         
;------------------------------------------------------------------------------- 
;        Increment RTC values 
;-------------------------------------------------------------------------------
Clock:        setc                             ;entry every second
              dadc.b SEC                       ;increment seconds, add carry bit
              cmp.b  #0x60,SEC                 ;one minute elapsed?
              jlo    Clockend                  ;no, return (C = 0)
              clr.b  SEC                       ;yes, clear seconds (C = 1)
Clock1:       dadc.b MIN                       ;increment minutes if set carry
              cmp.b  #0x60,MIN                 ;sixty minutes elapsed?
              jlo    Clockend                  ;no, jump to end of routine
              clr.b  MIN                       ;yes, clear minutes
              dadc.b HOUR                      ;increment hour
              cmp.b  #0x13,HOUR	               ;12 hours elapsed?	 
              jlo    Clockend                  ;no, no jump to end of routine
              mov.b  #0x01,HOUR                ;yes, set hours back to 1
Clockend:     ret

;-------------------------------------------------------------------------------
Setup:      ;Configure modules and control registers 
;-------------------------------------------------------------------------------
StopWDT:      mov    #WDTPW+WDTHOLD,&WDTCTL    ;stop Watchdog Timer
              bis.b  #WDTIE,IE1                ;enable WDT interrupt - used
                                               ;in ISRs
SetupLCD:     mov.b  #0x55,&LCDCTL             ;3mux LCD, segs20-23 = outputs
              bis.b  #0xFC,&P5SEL              ;set Rxx and COM pins for LCD
SetupBT:      mov.b  #BT_ADLY_1000,&BTCTL      ;Basic Timer interrupt = 1 second
              bic.b  #BTFRFQ1+BTFRFQ0,&BTCTL   ;set freqLCD = ACLK / 256
              mov.b  #BTIE,&IE2                ;enable Basic Timer interrupt
              
SetupFLL2:    bis.b  #XCAP18PF,&FLL_CTL0       ;set load capacitance for xtal              

              mov    #10000,R15
FLL_Wait:     dec    R15                       ;delay for FLL to lock, only 
              jnz    FLL_Wait		       ;needed at initial power up

SetupP1:      mov.b  #SetSW+ModeSW,&P1IE       ;P1 interrupts enabled
              bis.b  #SetSW+ModeSW,&P1IES      ;P1 edges select = high to low
              mov.b  #0x83,&P1DIR              ;outputs = P1.0,1,7
              bis.b  #Ref,&P1DIR               ;change Ref pin to output to hold
                                               ;inputs low - uses less power
                                               ;then to leave floating
              clr.b  &P1OUT                    ;clear output data
              
SetupCA:      mov.b  #P2CA0,&CACTL2            ;+Comp = P1.6
              
SetupPx:      mov.b  #0xFF,&P2DIR              ;set port to outputs
              clr.b  &P2OUT                    ;clear port 2 output data 
              mov.b  #0xFF,&P3DIR              ;set port to outputs
              mov.b  #0xFF,&P4DIR              ;set port to outputs 
              mov.b  #0xFF,P6DIR               ;set port to outputs

ClearRAM:     clr    SEC                       ;word operation clears SEC & MIN
                                               ;since are in consecutive bytes 
              mov    #0x012,HOUR               ;Clear FLAG, load 12 to HOURs
              clr    Index                     ;Reset mode index to 0

              clr.b  &P1IFG                    ;clear port 1 flags
              clr.b  &IFG1                     ;clear flag register 1
              clr.b  &IFG2                     ;clear flag register 1 
              eint                             ;enable interrupts	  

TestTiming:   mov.b  #4,TestFreq               ;load counter to only test temp
                                               ;once per 4 seconds
              mov.b  #10,AutoSW                ;load seconds to stay in time 
                                               ;mode before switches back to 
                                               ;thermostat mode                                               

SetupFlash:   xor    #FXKEY+LOCK,&FCTL3        ;unlock FLASH for write
              xor    #FXKEY+WRT,&FCTL1         ;enable FLASH write

SetPntr:      mov    #0x1000,Pointer           ;initialize pointer to first
                                               ;address in flash info memory
                                               
;;;              mov    #0x107B,Pointer           ;initialize pointer to 5 bytes
;;;                                               ;before segment boundry, use
;;;                                               ;this line for demo to write
;;;                                               ;& erase across boundry.
;;;                                               ;comment out standard lines. 
;;;                                               ;change 4 places, find ";;;"

              cmp.b  #0xFF,0(Pointer)          ;first info flash location empty?
              jnz    InitPntr                  ;no, jump to search for location
              mov.b  #0x79,0(Pointer)          ;yes, load initial setpoint value 
              jmp    InitSP                    ;jump to initialize setpoint

InitPntr:     cmp.b  #0xFF,1(Pointer)          ;next location empty?
              jz     InitSP                    ;yes, jump to initialize setpoint
              inc    Pointer                   ;no, increment pointer value
;;;              cmp    #0x1082,Pointer           ;pointer at end of 8 bytes?
;;;                                               ;use this line for demo to 
;;;                                               ;write & erase across boundry 
;;;                                               ;Change 4 places see ";;;"

              cmp    #0x10FF,Pointer           ;pointer at end of info memory?
                                               ;above line used in application
              jz     InitSP                    ;yes, setpoint = last location
              jmp    InitPntr                  ;loop to test next location

InitSP:       mov.b  0(Pointer),SetPnt         ;initialize setpoint

LockFl:       xor    #FXKEY+LOCK,&FCTL3        ;lock FLASH memory  

ClearLCD:     mov    #15,R15                   ;15 LCD memory bytes to clear
Clear1:       mov.b  #0,LCDM1-1(R15)           ;write zeros in LCD RAM locations
                                               ;'-1' to get LCDM1(0) LCD memory
              dec    R15                       ;all LCD mem clear?
              jnz    Clear1                    ;more LCD mem to clear go
                                                           
              ret

;-------------------------------------------------------------------------------
;	 P1 ISR: decide which input, then change mode or enable WDT as one-shot 
;-------------------------------------------------------------------------------
P1x_ISR:      bit.b  #4,&P1IFG                 ;test if mode switch set
              jnz    P12_ISR                   ;yes, jump to mode routine

P13_ISR:      mov    #WDT_ADLY_250+WDTNMIES,&WDTCTL ;WDT = 250ms interval,
              clr.b  &P1IFG                    ;clear all P1.x interrupts
              bic.b  #SetSW,&P1IE              ;disable P1.3 interrupt
              jmp    EndISR
                            
P12_ISR:      mov    #WDT_ADLY_250+WDTNMIES,&WDTCTL ;WDT = 250ms interval,
              bis.b  #LCDSON,&LCDCTL           ;ensure LCD display is on
              clr.b  &P1IFG                    ;clear all P1.x interrupts
              incd   Index                     ;double increment branch index
              cmp    #4,Index                 ;leaving setpoint mode?
              jz     StoreSP                   ;yes, jump to store value
              cmp    #8,Index                 ;out of range of branch table?
              jl     EndISR                    ;jump if still in range
              clr    Index                     ;clear index = Thermostat mode
EndISR:       bic.b  #WDTIFG,&IFG1             ;clear pending WDT interrupts              
              reti      

;-------------------------------------------------------------------------------
;        Routine to store setpoint and erase flash segement if necessary
;-------------------------------------------------------------------------------
StoreSP:      xor    #FXKEY+LOCK,&FCTL3        ;unlock for FLASH write or erase
              cmp.b  SetPnt,0(Pointer)         ;curr setpoint = value @ pointer?
              jz     LockFlash                 ;yes, no need to store value
              
;;;              cmp    #1082h,Pointer            ;pointer at end of 8 bytes of
;;;                                               ;info memory? Use this line 
;;;                                               ;for demo to write & erase 
;;;                                               ;across segment boundry. 
;;;                                               ;Change 4 places, see ";;;"

              cmp    #10FFh,Pointer            ;pointer at end of info memory?
                                               ;- use above line in application
              jz     EraseFlash                ;yes, erase info flash then write
              
SaveSP:       inc    Pointer                   ;increment pointer to new value
              mov    #FWKEY+WRT,&FCTL1         ;enable FLASH write
              mov.b  SetPnt,0(Pointer)         ;save new setpoint value to flash
TestBusy1:    bit    #BUSY,&FCTL3              ;check busy flag
              jnz    TestBusy1                 ;loop until not busy 
LockFlash:    xor    #FXKEY+LOCK,&FCTL3        ;lock FLASH memory  
              jmp    EndISR                    ;jump to end watchdog timer isr

EraseFlash:   mov    #FWKEY+ERASE,&FCTL1       ;set up for erase 
              clr.b  0(Pointer)                ;perform dummy write into segment
TestBusy2:    bit    #BUSY,&FCTL3              ;check busy flag
              jnz    TestBusy2                 ;loop until not busy 
              sub    #081h,Pointer             ;set pointer into first info
                                               ;segment
              mov    #FWKEY+ERASE,&FCTL1       ;set up for erase
              clr.b  0(Pointer)                ;perform dummy write into segment              
TestBusy3:    bit    #BUSY,&FCTL3              ;check busy flag
              jnz    TestBusy3                 ;loop until not busy 
              
;;;              mov    #0x107A,Pointer           ;initialize pointer to 6 bytes
;;;                                               ;before segment boundry, due 
;;;                                               ;to increment before store.
;;;                                               ;use this line for demo to 
;;;                                               ;write & erase across boundry.
;;;                                               ;change 4 places, see ";;;"
              
              mov    #0xFFF,Pointer            ;reset pointer to one less than
                                               ;segment start due to increment 
                                               ;before store
              jmp    SaveSP                    ;jump to store setpoint               

;-------------------------------------------------------------------------------
;	 Watchdog isr: WD timer expired, clear LCD, check switches, if set 
;        switch is pressed then increment time display
;-------------------------------------------------------------------------------
WDTInt:       bic    #LPM3,0(R1)               ;system active upon return
              cmp.b  #2,Index                  ;in setpoint mode?
              jeq    ChngSP                    ;yes, jump to change setpoint
              call   #ClearLCD                 ;clear LCD memory
              cmp.b  #4,Index                  ;in time mode?
              jne    ClrSW                     ;no, jump to clear switch

SetCLK:       bit.b  #SetSW,&P1IN              ;SET switch pressed?
              jc     ClrSW                     ;no, jump to clear switches
Set_2:        call   #Clock                    ;call routine to update clock
              call   #Dis_Clock                ;call routine to display clock
              mov    #0500,R15                 ;load delay
Set_1:        dec    R15                       ;decrement delay value
              jnz    Set_1                     ;loop to delay if not = 0
              jmp    SetCLK                    ;jump to check SET switch again
              
ChngSP:       bit.b  #SetSW,&P1IN              ;SET switch pressed?
              jc     ClrSW                     ;no, jump to clear switches
              cmp.b  #0x99,SetPnt               ;setpoint at max?
              jc     ResetSP                   ;yes, jump to reset to 32F
              dadd   #1,SetPnt                 ;no, decimal increment by 1
              jmp    SPdelay                   ;skip next instruction 
ResetSP:      mov    #0x32,SetPnt               ;reset setpoint to 32F

SPdelay:      mov    SetPnt,R14                ;R14 used to pass value
              call   #Display2                 ;update LCD display
              mov    #40000,R15                ;load delay
Set_3:        dec    R15                       ;decrement delay value
              jnz    Set_3                     ;loop to delay if not = 0
              jmp    ChngSP                    ;jump to check SET switch again

ClrSW:        clr.b  &P1IFG                    ;clear Port1 interrupts flags
                                               ;in case of switch bounce
              bis.b  #SetSW,&P1IE              ;enable P1.3 Interrupt
Next1:        mov    #WDTPW+WDTHOLD,&WDTCTL    ;stop Watchdog Timer
WDTIend:      reti

;-------------------------------------------------------------------------------
;        Basic Timer isr:  BT causes 1 second interrupt. Mode bits changed on
;        stack so CPU is returned in active upon return from the interrupt
;-------------------------------------------------------------------------------
BT_ISR:       bic    #LPM3,0(R1)               ;clear SR low power mode bits
                                               ;from top of stack
              call   #Clock                    ;update hours and minutes each
                                               ;second even if not displayed
              reti

;---------------------------------------------------------------------------
TAX_ISR:      ;ISR for CCR1
;---------------------------------------------------------------------------
CCR1_ISR:     bic    #LPM0,0(R1)               ;exit LPM0 on reti
              bic    #0x0001,&CCTL1            ;clear CCR1 interrupt flag
              reti                             
              		 
;-------------------------------------------------------------------------------
;        Define characters for #VIM-503 4.5 digit 3 multiplex LCD 
;-------------------------------------------------------------------------------
            .equ    f,0x001
            .equ    e,0x002
            .equ    h,0x004
            .equ    a,0x010
            .equ    g,0x020
            .equ    d,0x040 
            .equ    b,0x100                     ;decimal point
            .equ    c,0x200
            .equ    y,0x400

            .align  1
LCD_Tab:    .word  a+b+c+d+e+f               ;displays "0"
            .word  b+c                       ;displays "1"
            .word  a+b+d+e+g	               ;displays "2"
            .word  a+b+c+d+g                 ;displays "3"
            .word  b+c+f+g                   ;displays "4"
            .word  a+c+d+f+g                 ;displays "5"
            .word  a+c+d+e+f+g               ;displays "6"
            .word  a+b+c                     ;displays "7"
            .word  a+b+c+d+e+f+g             ;displays "8"
            .word  a+b+c+d+f+g               ;displays "9"
            .word  g                         ;displays "-"

;-------------------------------------------------------------------------------
;        Resistance table, K ohms to 99-59 degrees F.  
;-------------------------------------------------------------------------------
              .align    1                      ;evenly aligned
RES_TAB:      .word  6446                      ;99 F
              .word  6570
              .word  6693
              .word  6817                      
              .word  6941                      ;95 F
              .word  7093
              .word  7245
              .word  7398
              .word  7550
              .word  7703                      ;90 F
              .word  7856
              .word  8008
              .word  8161
              .word  8313
              .word  8500                      ;85 F
              .word  8688
              .word  8875
              .word  9063
              .word  9250
              .word  9438                      ;80 F
              .word  9625
              .word  9813
              .word  10000
              .word  10232
              .word  10464                     ;75 F
              .word  10697
              .word  10929
              .word  11161
              .word  11393
              .word  11626                     ;70 F
              .word  11858
              .word  12090
              .word  12378
              .word  12666
              .word  12953                     ;65 F
              .word  13241
              .word  13529
              .word  13817
              .word  14104
              .word  14392                     ;60 F
              .word  14680
              .word  15044
              .word  15409
              .word  15773
              .word  16137                     ;55 F		 
              .word  16502
              .word  16867
              .word  17231
              .word  17596
              .word  17960                     ;50 F
              .word  18414
              .word  18869
              .word  19323
              .word  19778		 
              .word  20232                     ;45 F
              .word  20687
              .word  21141
              .word  21596
              .word  22050
              .word  22631                     ;40 F
              .word  23212
              .word  23793
              .word  24374
              .word  24956
              .word  25527                     ;35 F
              .word  26118
              .word  26699
              .word  27280                     ;32 F

;-------------------------------------------------------------------------------
;        41x Interrupt vectors
;-------------------------------------------------------------------------------
              .section  .vectors
              .word  BT_ISR                    ;Basic Timer
              .word  RESET                     ;I/O Port 2
              .word  RESET                     ;no source
              .word  RESET                     ;no source
              .word  P1x_ISR                   ;I/O Port 1
              .word  TAX_ISR                   ;Timer A3 CCIFG1 & 2
              .word  RESET                     ;Timer A3 CCIFG0
              .word  RESET                     ;no source
              .word  RESET                     ;no source
              .word  RESET                     ;no source
              .word  WDTInt                    ;Watchdog Timer, timer mode
              .word  RESET                     ;Comparator A
              .word  RESET                     ;no source
              .word  RESET                     ;no source
              .word  RESET                     ;NMI, osc. fault, ACCVIFG
              .word  RESET                     ;POR, ext. reset, Watchdog
