;******************************************************************************
; File:		water_meter.s43
;
; Description:	Water meter demo using TI patented sensor. This code run on MSP430F41x.
;
; Revisions:	1.0	27 Jun 2001
;
; Copyright:	Texas Instruments Hong Kong Ltd
;
;   Kes Tam
;   Texas Instruments Hong Kong
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;   $Id: water_meter.S,v 1.2 2004/06/23 19:16:44 cliechti Exp $
;
;******************************************************************************
;
; Note to users:
;
; THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR
; REPRESENTATIONS, EITHER EXPRESS, IMPLIED OR STATUTORY, 
; INCLUDING ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS 
; FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY OR 
; COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE. 
; TI DISCLAIMS ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET 
; POSSESSION, AND NON-INFRINGEMENT OF ANY THIRD PARTY 
; INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR 
; YOUR USE OF THE PROGRAM.
;
; IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL, 
; CONSEQUENTIAL OR INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY 
; THEORY OF LIABILITY AND WHETHER OR NOT TI HAS BEEN ADVISED 
; OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY OUT 
; OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM. 
; EXCLUDED DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF 
; REmovAL OR REINSTALLATION, COMPUTER TIME, LABOR COSTS, LOSS 
; OF GOODWILL, LOSS OF PROFITS, LOSS OF SAVINGS, OR LOSS OF 
; USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S 
; AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF 
; YOUR USE OF THE PROGRAM EXCEED FIVE HUNDRED DOLLARS 
; (U.S.$500).
;
; Unless otherwise stated, the Program written and copyrighted 
; by Texas Instruments is distributed as "freeware".  You may, 
; only under TI's copyright in the Program, use and modify the 
; Program without any charge or restriction.  You may 
; distribute to third parties, provided that you transfer a 
; copy of this license to the third party and the third party 
; agrees to these terms by its first use of the Program. You 
; must reproduce the copyright notice and any other legend of 
; ownership on each copy or partial copy, of the Program.
;
; You acknowledge and agree that the Program contains 
; copyrighted material, trade secrets and other TI proprietary 
; information and is protected by copyright laws, 
; international copyright treaties, and trade secret laws, as 
; well as other intellectual property laws.  To protect TI's 
; rights in the Program, you agree not to decompile, reverse 
; engineer, disassemble or otherwise translate any object code 
; versions of the Program to a human-readable form.  You agree 
; that in no event will you alter, remove or destroy any 
; copyright notice included in the Program.  TI reserves all 
; rights not specifically granted under this license. Except 
; as specifically provided herein, nothing in this agreement 
; shall be construed as conferring by implication, estoppel, 
; or otherwise, upon you, any license or other right under any 
; TI patents, copyrights or trade secrets.
;
; You may not use the Program in non-TI devices.
;
;******************************************************************************
;
; Development history :
;
;   6/27/2001   Successfully, converting the code from Christian Hernitscheck,
;               it uses sensor IP invented by Heinrich Pinta.
;   8/10/2001   add load cap control to x'tal
;               Change Vref to Vcc/2
;
;******************************************************************************


;-----------------------------------------------------------------------------
; Compilation control
;-----------------------------------------------------------------------------
	.equ	COUNT_UP_ONLY,0	;1=> meter only do count up in either direction
				;0=> meter count up or down depending on the direction

;-----------------------------------------------------------------------------
; include available peripheral port addresses
;-----------------------------------------------------------------------------
#include <io.h>

;-----------------------------------------------------------------------------
; Pin assignment
;-----------------------------------------------------------------------------
;
;           MSP430F413
;         ---------------
;     .  |               |
;    /|\ |           XIN |---
;     |  |               |    32768Hz
;     ---| RST      XOUT |---
;        |               |
;        |               |   
;        |           R33 |____
;        |               |    /
;        |               |    \ 1M
;        |           R32 |____/
;        |               |    /
;        |               |    \ 1M
;        |           R31 |____/
;        |               |    /
;        |               |    \ 1M
;        |           R30 |____/
;        |               |    |
;        |               |   GND
;        |               |
;        |               |          ____________LCD__T218010___________
;        |          COM3 |---------|2 COM4      __    __    __    __   |
;        |          COM2 |---------|1 COM3     |__|  |__|  |__|  |__|  |
;        |          COM1 |---------|3 COM2     |__|  |__|  |__|  |__|  |
;        |               |    _____|20 COM1     __    __    __    __   |
;        |          COM0 |___|_____|4  COM1                            |
;        |               |         |  S0            S14                |
;        |               |         |__19____________5__________________|
;        |               |            |             |
;        |          S0   |------------              |
;        |           :   |   :                      |
;        |           :   |   :                      |
;        |          S14  |-------------------------- 
;        |               |
;        |          P1.1 |---> sensor1_GND
;        |               |
;        |          P1.5 |---> sensor2_GND
;        |               |
;        |          P1.7 |---> sensor3_GND
;        |               |
;        |      P1.0/TA0 |---> osc_trigger
;        |               |
;        |      P1.6/CA0 |---> detect_in
;        |               |
;         ---------------
;
;
;-----------------------------------------------------------------------------

           	.equ	bSensor1,0x02	;Port bit of Sensor1
        	.equ	bSensor2,0x20	;Port bit of Sensor2
        	.equ	bSensor3,0x80	;Port bit of Sensor3

;*******************************************************************************
;*****                                                                     *****
;***** Working flow                                                        *****
;*****                _                                     P1.1 P1.5 P1.7 *****
;*****  P1.0/TA0 ____| |___________________________________   L    Z    Z  *****
;*****                           _                                         *****
;*****  P1.0/TA0 _______________| |________________________   Z    L    Z  *****
;*****                                      _                              *****
;*****  P1.0/TA0 __________________________| |_____________   Z    Z    L  *****
;*****           ________    _______    _______    ________                ***** 
;*****  P1.6/CA0         \/\/       \/\/       \/\/         Comp.A input   *****
;*****                                                                     *****
;*******************************************************************************

;-----------------------------------------------------------------------------
; Extern defined symbols (constants, variables, subroutines)
;-----------------------------------------------------------------------------
	        .extern	LCD_Tab
	        .extern	LCD_sensor_pos

;-----------------------------------------------------------------------------
; Constant Definitions
;-----------------------------------------------------------------------------
        	.equ	COUNT_DIVIDER,3	;0,1=> no divider
					;2 or above => scale down the actual count displayed on the meter
					;note : one count for 120 deg of rotation

		.equ	TimeOut,50	    ;defines sample frequency

	        .equ	PowerDownCount,64   ;Value to recuse scan time after x times the same state have been detected

	        .equ	HighSample,0x17	    ;ACLK
					;32768/256 = 128Hz Interrupt Frequency
					;LCD frame frequency @4Mux: 64Hz

        	.equ	LowSample,0x33	    ;ACLK
					;32768/256/8 = 8Hz Interrupt Frequency
					;LCD frame frequency @4Mux: 64Hz


;******************************************************************************
; Definitions of CPU registers
;******************************************************************************
#define		Register	r5
#define		temp_reg	r6
#define		adj_count	r7
#define		sensor_in	r8
#define		last_sensor_in	r9
#define		last_falling	r10
#define		flow_count	r11

#if (COUNT_DIVIDER > 1)
#define		meter_count	r12
#else
#define		meter_count	flow_count
#endif

#define		flow_idle_count	r13
#define		SensorCounter	r14

;******************************************************************************
; Definitions of variables in the RAM
;******************************************************************************
        .section    .data
dummy:	.ds	1

;******************************************************************************
; M A I N   P R O G R A M
;******************************************************************************
	.section    .text
Main:	mov	#0x300,R1		;initialization of Stack Pointer
	mov	#WDTPW+WDTHOLD,&WDTCTL	;disable Watchdog                

;-----------------------------------------------------------------------------
; init FLL
;-----------------------------------------------------------------------------
	mov.b	#XCAP10PF,&FLL_CTL0

;-----------------------------------------------------------------------------
;init Ports
;-----------------------------------------------------------------------------

;init unused Ports
	mov.b	#0x01,&P2OUT		;P2.1-7=0, P2.0=0 
	mov.b	#0xFE,&P2DIR		;P2.1-7=output, P2.1 =input
	mov.b	#0x00,&P3OUT		;P3.x=0
	mov.b	#0xFF,&P3DIR		;P3.x=output, P4.- =input
	mov.b	#0x00,&P4OUT		;P4.x=0
	mov.b	#0xFF,&P4DIR		;P4.x=output, P4.- =input
	mov.b	#0x00,&P5OUT		;P5.x=0
	mov.b	#0xFF,&P5DIR		;P5.x=output, P5.- =input
	mov.b	#0x00,&P6OUT		;P6.x=0
	mov.b	#0xFF,&P6DIR		;P6.x=output, P6.- =input

InitPorts:
	mov.b #0xFC,&P5SEL		;Enable R(LCD) + COM Lines
	mov.b #0x00,&P1OUT		;P1.0/1/5/7="L", P1.2/3/4/6="L"
	mov.b #0xBF,&P1DIR		;P1.0/1/2/3/4/5/7=output, P1.6=input
	mov.b #0x01,&P1SEL		;P1.0 => module function (TA0 output)

;-----------------------------------------------------------------------------
;init Varibles
;-----------------------------------------------------------------------------
	clr	last_sensor_in
	mov	#7,last_falling	        ;assume all sensor has gone through damping
					;this will give no counting in the first rotation
	clr	SensorCounter
	mov	#PowerDownCount,flow_idle_count


;Startup delay loop
	dint
	push	r15
	clr	r15
StrtLoop:
	dec	r15
	jnz	StrtLoop
	pop	r15


;-----------------------------------------------------------------------------
; Init LCD
;-----------------------------------------------------------------------------
LCD_On: 
	mov.b	#(LCDON+LCD4MUX+LCDP2),&LCDCTL	;STK LCD 4Mux, S0-S23


;-----------------------------------------------------------------------------
; Init Basic Timer
;-----------------------------------------------------------------------------
InitBasicTimer:
	mov.b	#HighSample,&BTCTL

;-----------------------------------------------------------------------------
; Init Timer_A
;-----------------------------------------------------------------------------
InitTimerA:
	mov	#TimeOut,&CCR0		;pulse length and sample frequency
	mov	#0x204,&TACTL		;CLOCK = SMCLK, Stop Mode, 
					;clear counter, interrupt disabled
	bis	#0x002,&TACTL		;interrupt enabled

	mov	#0x000,&CCTL0		;Output = 0
	mov	#0x0060,&CCTL0		;Output Mode=PWM set/REset, compare
					;    mode, interrupt disabled
	mov	#0x9100,&CCTL1		;CC block 1 is used for measurement, capture
					;    mode, negative edge, CAOUT input, interrupt disabled

;-----------------------------------------------------------------------------
; Init Comp_A
;-----------------------------------------------------------------------------
InitCompA:
	mov.b	#0x60,&CACTL1		;Vref=Vcc/2, interrupt disabled, CAON=0
	mov.b	#0x4,&CACTL2		;P1.6/CA0 is used as comparator input

	clr	flow_count
#if (COUNT_DIVIDER > 1)
	clr	meter_count
#endif

	call	#clearLCD		;clear LCD
	clr	r5
	call	#Display

;-----------------------------------------------------------------------------
; start interrupt here
;-----------------------------------------------------------------------------
	bic.b	#BTIFG,&IFG2
	bis.b	#BTIE,&IE2		;enable basic timer interrupt       

	eint

;-----------------------------------------------------------------------------
; Idle loop
;-----------------------------------------------------------------------------

MainLoop:
	bis	#(CPUOFF+SCG0+SCG1),R2	;Low Power Mode   
          
	jmp	MainLoop

;******************************************************************************
; S U B R O U T I N E S
;******************************************************************************

;-----------------------------------------------------------------------------
; clearLCD : clear LCD, i.e. no display
;-----------------------------------------------------------------------------
clearLCD:
	mov	#0x1A,Register	        ;clear display memory

clearLCD1:
	mov.b	#0,(LCDMEM-1)(Register)
	dec	Register
	jnz	clearLCD1
	ret

;-----------------------------------------------------------------------------
; rrax4 ; rotate byte by 4 bits
;-----------------------------------------------------------------------------
rrax4:rra	Register		;Nibble Rotate Right
	rra	Register
	rra	Register
	rra	Register
	ret

;------------------------------------------------------------------------
; Display : display 4 digits in LCD
; input = r5
;------------------------------------------------------------------------
Display:
	call	#BinDec			;r5 contains Binary Number to display
	push	r5
	and	#0xF,r5
	mov.b	LCD_Tab(r5),&LCDM3
	pop	r5
	call	#rrax4
	push	r5
	and	#0xF,r5
	mov.b	LCD_Tab(r5),&LCDM4
	pop	r5
	call	 #rrax4
	push	r5
	and	#0xF,r5
	mov.b	LCD_Tab(r5),&LCDM5
	pop	r5
	call	#rrax4
	and	#0xF,r5
	mov.b	LCD_Tab(r5),&LCDM6
	ret

;------------------------------------------------------------------------
; BinDec : convert binary data to BCD
;------------------------------------------------------------------------
BinDec:
	push	r4			;r5 contains Binary Number
	push	r6
	push	r7
	mov	r5,r4
	mov	#16,r7
	clr	r6
	clr	r5
BinDec1:
	rla	r4
	dadd	r5,r5
	dadd	r6,r6
	dec	r7
	jnz	BinDec1
	pop	r7
	pop	r6
	pop	r4
	ret          

;******************************************************************************
; I N T E R R U P T   S E R V I C E   R O U T I N E S
;******************************************************************************


;------------------------------------------------------------------------
; ISR_TimA : timer A interrupt service routine for overflow and CCR1, 2,..
;------------------------------------------------------------------------
ISR_TimA:
	add   &TAIV,R0
ISR_End:
	reti

	jmp	ISR_CCIFG1
	jmp	ISR_CCIFG2
	jmp	ISR_End
	jmp	ISR_End
	jmp	NextSensor		;ISR_TAIFG
;
ISR_TAIFG:
ISR_CCIFG1:
ISR_CCIFG2:
	reti

;------------------------------------------------------------------------

NextSensor:
	incd	SensorCounter
	add	SensorCounter,R0
	reti

	jmp	ISR_Sensor1
	jmp	ISR_Sensor2
	jmp	ISR_Sensor3  
	jmp	ISR_SensorError

ISR_SensorError:
	clr	SensorCounter
	reti

;------------------------------------------------------------------------
ISR_Sensor1:
	bit	#COV,&CCTL1		;Test if overflow -> more then one pulse
	rlc	sensor_in
	mov	&CCR1,temp_reg		;dummy read to start overflow detection
	bic	#(COV+CCIFG),&CCTL1
	xor.b	#(bSensor1+bSensor2),&P1DIR
					;Sensor 2 0n
	reti

;------------------------------------------------------------------------
ISR_Sensor2:
	bit	#COV,&CCTL1		;Test if overflow -> more then one pulse
	rlc	sensor_in
	mov	&CCR1,temp_reg		;dummy read to start overflow detection
	bic	#COV+CCIFG,&CCTL1
	xor.b	#bSensor2+bSensor3,&P1DIR
					;Sensor 3 on
	reti
;------------------------------------------------------------------------
ISR_Sensor3:         
	bit	#COV,&CCTL1		;Test if overflow -> more then one pulse
	rlc	sensor_in
	bic	#0x10,&TACTL		;Stop Mode

	bic.b	#CAON,&CACTL1		;switch off Comparator_A


	bis.b	#(bSensor1+bSensor2+bSensor3),&P1DIR
					;keep all sensors on
					;this avoids floating input lines

;----------------------------------------------------
; check metal plate entering the inductor,
; sensor will go from undamped to damped,
; i.e. sensor input bit falling from 1 to 0
;----------------------------------------------------
do_flow_process:
	mov	sensor_in,temp_reg
	inv	temp_reg
	and	last_sensor_in,temp_reg
	mov	sensor_in,last_sensor_in	;backup
	and	#7,temp_reg		;keep only defined sensor bits

#if 1	/* 1=>auto reduce sampling rate if idle too long */
					;0=>always use fixed sampling rate

	jnz	set_high_sampling	;jump if any sensor bit falling
	dec	flow_idle_count
	jge	done_flow_process
	mov	#PowerDownCount,flow_idle_count
	mov.b	#LowSample,&BTCTL
	jmp	done_flow_process

set_high_sampling:
	mov	#PowerDownCount,flow_idle_count
	mov.b	#HighSample,&BTCTL
done_sampling_adj:

#else
	jz	done_flow_process	;jump if no sensor bit falling
#endif

;----------------------------------------------------
check_sensor_jitter:

;check same sensor falling due to jittering,
;this could happen when the metal plate cover only half inductor and stand still
;(Note : two sensors failed will give same result, but assuming this will not happen)

	bit	temp_reg,last_falling	;same sensor?
	jnz	update_last_falling	;do nothing if same sensor

;----------------------------------------------------
;update rotation icon to indicate wheel rolling

	mov.b	LCD_sensor_pos(temp_reg),&LCDM1
					;update falling sensor position in the LCD

;----------------------------------------------------
#if COUNT_UP_ONLY
	inc	flow_count
	bit	last_falling,sensor_in	;last falling sensor still on the plate?
	jz	done_count_update
	inc	flow_count		;one sensor failed and not detected,
					;so need one more count
done_count_update:

#else

;--do up and down counting here--

;assume no sensor fail first
	mov	#1,adj_count
	bit	last_falling,sensor_in	;last falling sensor still on the plate?
	jz	do_bit_rotation
	
;one sensor failed, need one more count adj and one more rotation
	inc	adj_count
	mov.b	rla_3_table(last_falling),last_falling
					;rotation of 3 bits

do_bit_rotation:
	mov.b	rla_3_table(last_falling),last_falling
					;rotation of 3 bits

chk_flow_dir:
	bit	last_falling,temp_reg	;if align, forward direction, else backward
	jnz	update_count
	inv	adj_count		;adj_count = -adj_count
	inc	adj_count

update_count:
	add	adj_count,flow_count
#endif

update_last_falling:
	mov	temp_reg,last_falling
	
;----------------------------------------------------
#if (COUNT_DIVIDER > 1)
chk_count_high:
	cmp	#COUNT_DIVIDER,flow_count
	jl	chk_count_low
	sub	#COUNT_DIVIDER,flow_count
	inc	meter_count
	jmp	done_chk_count

chk_count_low:
	cmp	#(-COUNT_DIVIDER+1),flow_count
	jge	done_flow_process
	add	#COUNT_DIVIDER,flow_count
	dec	meter_count
done_chk_count:
#endif

;----------------------------------------------------
check_range_overflow:
	cmp	#10000,meter_count
	jl	check_range_underflow
	sub	#10000,meter_count

check_range_underflow:
	tst	meter_count
	jge	done_check_range
	add	#10000,meter_count
done_check_range:

;----------------------------------------------------
update_display:
	mov	meter_count,r5
	call	#Display

;----------------------------------------------------

done_flow_process:
;----------------------------------------------------

	bis	#LPM3,0(R1)		;manipulate status register on stack
                                        ;   => Main program -> Low Power Mode 3

	reti

;----------------------------------------------------
rla_3_table:
	.byte	0x00
	.byte	0x02
	.byte	0x04
	.byte	0x06
	.byte	0x01
	.byte	0x03
	.byte	0x05
	.byte	0x07

;----------------------------------------------------
; ISR_BT1: : Basic Timer interrupt servicing routine
;----------------------------------------------------
ISR_BT1:
	bis.b	#(bSensor1+bSensor2+bSensor3),&P1DIR
					;switch all Sensors on
	bic.b	#bSensor2+bSensor3,&P1DIR
					;keep only first Sensor on

	clr	SensorCounter
	clr	sensor_in
	mov	#(TimeOut/2),&TAR	;only half cycle to start first measurement
	mov	#(LPM0+GIE),0(R1)	;manipulate status register on stack

	bis.b	#CAON,&CACTL1		;switch on Comparator_A
	mov	&CCR1,temp_reg		;dummy read to start overflow detection
	bic	#COV+CCIFG,&CCTL1
	bis	#0x10,&TACTL		;Up to CCR0 Mode / no clear

	reti

;******************************************************************************
; Interrupt Vector Table
;******************************************************************************
	.section    .vectors		;*** Interrupt Vector Table MSP430F41x
	.word	ISR_BT1			;FFE0h: Basic Timer1
	.word	Main			;FFE2h: I/O Port 2
	.word	Main			;FFE4h: not used
	.word	Main			;FFE6h: not used
	.word	Main			;FFE8h: I/O Port 1
	.word	ISR_TimA		;FFEAh: Timer_A (CCIFG1, CCIFG2, TAIFG)
	.word	Main			;FFECh: Timer_A (CCIFG0)
	.word	Main			;FFEEh: not used
	.word	Main			;FFF0h: not used
	.word	Main			;FFF2h: not used
	.word	Main			;FFF4h: Watchdog Timer
	.word	Main			;FFF6h: Comparator_A
	.word	Main			;FFF8h: not used
	.word	Main			;FFFAh: not used
	.word	Main			;FFFCh: NMI, Osc. Fault, Flash memory
	.word	Main			;FFFEh: Power-Up, ext. Reset, Watchdog,Flash mem.
        
;******************************************************************************
