/************************************************************
* THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR
* REPRESENTATIONS, EITHER EXPRESS, IMPLIED OR STATUTORY, 
* INCLUDING ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS 
* FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY OR 
* COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE. 
* TI DISCLAIMS ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET 
* POSSESSION, AND NON-INFRINGEMENT OF ANY THIRD PARTY 
* INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR 
* YOUR USE OF THE PROGRAM.
*
* IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL, 
* CONSEQUENTIAL OR INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY 
* THEORY OF LIABILITY AND WHETHER OR NOT TI HAS BEEN ADVISED 
* OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY OUT 
* OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM. 
* EXCLUDED DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF 
* REMOVAL OR REINSTALLATION, COMPUTER TIME, LABOR COSTS, LOSS 
* OF GOODWILL, LOSS OF PROFITS, LOSS OF SAVINGS, OR LOSS OF 
* USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S 
* AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF 
* YOUR USE OF THE PROGRAM EXCEED FIVE HUNDRED DOLLARS 
* (U.S.$500).
*
* Unless otherwise stated, the Program written and copyrighted 
* by Texas Instruments is distributed as "freeware".  You may, 
* only under TI's copyright in the Program, use and modify the 
* Program without any charge or restriction.  You may 
* distribute to third parties, provided that you transfer a 
* copy of this license to the third party and the third party 
* agrees to these terms by its first use of the Program. You 
* must reproduce the copyright notice and any other legend of 
* ownership on each copy or partial copy, of the Program.
*
* You acknowledge and agree that the Program contains 
* copyrighted material, trade secrets and other TI proprietary 
* information and is protected by copyright laws, 
* international copyright treaties, and trade secret laws, as 
* well as other intellectual property laws.  To protect TI's 
* rights in the Program, you agree not to decompile, reverse 
* engineer, disassemble or otherwise translate any object code 
* versions of the Program to a human-readable form.  You agree 
* that in no event will you alter, remove or destroy any 
* copyright notice included in the Program.  TI reserves all 
* rights not specifically granted under this license. Except 
* as specifically provided herein, nothing in this agreement 
* shall be construed as conferring by implication, estoppel, 
* or otherwise, upon you, any license or other right under any 
* TI patents, copyrights or trade secrets.
*
* You may not use the Program in non-TI devices.
*************************************************************/

/****************************************************************/
/* FILENAME: main.c                            		        */
/* DESCRIPTION: This program uses the MSP430F149 to read 500 	*/
/*    samples from the TLC2552 12-bit 200KSPS ADC.              */  
/*    The samples are stored in the buffers CH0_data & CH1_data.*/
/*    Refer to the .map file created by the compiler for the    */  
/*    address in memory for CH0_data & CH1_data.                */  
/*    AUTHOR: Joe Purvis, Data Acquisition Products,            */  
/*            Dallas Texas                                      */
/*    CREATED 2003(C) BY TEXAS INSTRUMENTS INCORPORATED.  	*/
/*    VERSION: 1.0 						*/
/*    Adapted for mspgcc by Steve Underwood <steveu@coppice.org>*/
/****************************************************************/
#include <signal.h>
#include <io.h>

/**************** Function prototypes ***********************************/
/*                                                                      */
void init_sys(void);                /* MSP430 Initialisation routine    */
void init_adc(void);                /* adc initialisation routine       */   
void delay(void);                   /* Software delay                   */
void complete(void);                /* Transmit / Receive ready         */
void convert(void);                 /* Do the conversions               */
void display(void);                 /* Indicate Program complete        */
/*                                                                      */            
/************************************************************************/

/************************************************************************/
/*                                                                      */
/* Global variable declarations                                         */
/*                                                                      */
/************************************************************************/

unsigned int    CH0_data[500];      // Storage for converted data
unsigned int    CH1_data[500];      // Storage for converted data

int   		CH0_buff, CH1_buff;
int             trash;

/************************************************************************/
/*                                                                      */
/* main() variable declarations                                         */
/*                                                                      */
/************************************************************************/

#define     CS          0x01

int main(void)
{ 
init_sys();                   	// Initialise the MSP430       

init_adc();                   	// Initialise the adc
                              
      do {
      convert();              	// Do conversions from each channel
      } while (CH0_buff < 500); // 

display();		      	// Flash the LED on the MSP430 when all the samples
			      	//  have been taken		
}			      	

/************************************************************/
/* Prototype - init_sys                                     */
/*                                                          */
/* !NA init_sys                                             */
/* !LA ANSI C                                               */
/* !PI *                                                    */
/* !PO *                                                    */
/* !LV *                                                    */
/*  Description                                             */
/*  This prototype initialises the MSP430F149               */
/************************************************************/
void init_sys(void)
{
void setupXT2(void);    /* Local function prototype         */ 
void setupports(void);  /* Local function prototype         */
void setupSPI(void);    /* Local function prototype         */    

setupXT2();       
setupports();
setupSPI();
}
/************************************************************/
/* Prototype - setupXT2                                     */
/*                                                          */
/* !NA XT2_osc                                              */
/* !LA ANSI C                                               */
/* !PI *                                                    */
/* !PO *                                                    */
/* !LV *                                                    */
/*  Description                                             */
/*  This prototype sets-up the XT2 oscillator and tests     */
/*  that it has settled before moving on                    */
/************************************************************/      
void setupXT2 (void)

{
WDTCTL = WDTPW + WDTHOLD;           // Stop watchdog timer  
BCSCTL1&=XT2OFF;                    // Switch on the XT2 osc.
BCSCTL2|=SELM1+SELS;                // Select XT2 osc. for 
                                    // SMCLK & MCLK                                    
                                    // Test the osc. flag bit
      do {
         IFG1 &= ~OFIFG;            // Clear the OFIFG bit
         } while (OFIFG&IFG1);      //        
}
                             

/************************************************************/
/* Prototype - setupports                                   */
/*                                                          */
/* !NA setupports                                           */
/* !LA ANSI C                                               */
/* !PI *                                                    */
/* !PO *                                                    */
/* !LV *                                                    */
/*  Description                                             */
/*  This prototype sets-up the GPIO ports as appropriate    */
/************************************************************/      
void setupports (void)
{    
P1DIR = BIT0;                       // P1.0 output for LED
P1OUT &= ~BIT0;                     // Clear P1.0
P3SEL = BIT3 + BIT2 + BIT1;         // Bits 3, 2 & 1 are assigned as SPI specific pins
P3DIR |= CS;                        // Set the CS bit as an output
P3OUT = CS;                         // De-assert CS for the adc - HIGH                             
}

/************************************************************/
/* Prototype - setupSPI                                     */
/*                                                          */
/* !NA setupSPI                                             */
/* !LA ANSI C                                               */
/* !PI *                                                    */
/* !PO *                                                    */
/* !LV *                                                    */
/*  Description                                             */
/*  This prototype sets-up the P3 for communication via SPI */
/************************************************************/      
void setupSPI (void)

/************************************************************/
/* System definitions                                       */
/************************************************************/
#define     SPIen       0x40

{    
ME1 |= SPIen;                       // Module Enable - SPI
                                    // 
U0CTL &= ~SWRST;                    // Make sure the RESET bit is off                                   
U0CTL |= CHAR + SYNC + MM;          // USART0 module operation
                                    // CHAR = 1 => 8-bit data 
                                    // SYNC = 1 => SPI selected
                                    // MM = 1 => master mode,
                                    //     MSP is the master

U0TCTL |= SSEL0 + SSEL1 + STC;      // USART0 Tranmit control register
                                    // SSEL0 = 1 & SSEL1 = 1
                                    // => SMCLK is used for baud-rate generation
                                    // STC = 1 => 3-pin SPI mode selected
U0BR0  = 0x02;                      // Divide SMCLK by 4 => transfer clock
U0BR1  = 0x00;                      //
U0MCTL = 0x00;                      // Modulation control - not used. Ensure 
                                    // all these bits are reset                                                           
}

/************************************************************/
/* Prototype - init_adc                                     */
/*                                                          */
/* !NA init_adc                                             */
/* !LA ANSI C                                               */
/* !PI *                                                    */
/* !PO *                                                    */
/* !LV *                                                    */
/*  Description                                             */
/*  This prototype sets-up the TLV2553 ADC                  */
/************************************************************/      
void init_adc (void)
{     
int i;
U0TXBUF = 0x00;                     // Start SCLK to ADC
complete();                         // 
trash = U0RXBUF;
				    // Reset the Multiplexer
				    //
P3OUT &= ~CS;                       // Assert CS* LOW
      U0TXBUF = 0x00;               // Start SCLK
      				    //	4SCLK =< CS* < 8SCLK
      for (i=0; i< 1; i++); 	    //  This depends on the Baud rate
				    //  the user sets. 
				    //  May be need to be altered.
P3OUT |= CS; 			    // De-assert CS* HIGH

trash = U0RXBUF;                    // De-assert CS HIGH
}

/************************************************************/
/* Prototype - complete                                     */
/*                                                          */
/* !NA complete                                             */
/* !LA ANSI C                                               */
/* !PI *                                                    */
/* !PO *                                                    */
/* !LV i                                                    */
/*  Description                                             */
/*  This prototype checks the URXIFG0-bit. If the receiver  */
/*  is still receiving data the program waits until all bits*/
/*  are received and the URXIFG0 bit is asserted            */
/************************************************************/
void complete(void)
{
      do
      {
      IFG1 &=~ URXIFG0;
      }
      while (URXIFG0 & IFG1);      
}                                          

/************************************************************/
/* Prototype - delay                                        */
/*                                                          */
/* !NA delay                                                */
/* !LA ANSI C                                               */
/* !PI *                                                    */
/* !PO *                                                    */
/* !LV i                                                    */
/*  Description                                             */
/*  This prototype gives a delay of around 1 second         */
/************************************************************/

void delay(void)
      {
      unsigned int i;
      for (i = 65000; i > 0; i--);
      }  


/************************************************************/
/* Prototype - flash                                        */
/*                                                          */
/* !NA flash                                                */
/* !LA ANSI C                                               */
/* !PI *                                                    */
/* !PO *                                                    */
/* !LV i                                                    */
/*  Description                                             */
/*  This prototype indicates the program is complete        */
/************************************************************/
void display(void)
      {
      while(1)
            {
            P1OUT |= BIT0;          // Set P1.0
            delay();
            P1OUT &= ~BIT0;         // Clear P1.0
            delay();
            }       
      }


/************************************************************/
/* Prototype - convert                                      */
/*                                                          */
/* !NA convert                                              */
/* !LA ANSI C                                               */
/* !PI *                                                    */
/* !PO *                                                    */
/* !LV *                                                    */
/*  Description                                             */
/*  This prototype does the adc conversion                  */
/************************************************************/      
void convert (void)
{
unsigned int hi_byte, lo_byte, temp;

					  // *** First conversion is trash 

if (CH0_buff == 0)
      {
      P3OUT &= ~CS;                       // Assert CS LOW
      U0TXBUF = 0x00;                     // Send clocks to the ADC, this shifts 
					  //  in the first 8 bits of data.
      complete();                         // Wait until all 8 bits have been received.
      hi_byte = U0RXBUF;		  // Store this data in the hi-byte variable.
      hi_byte = hi_byte << 8;             // Left-shift the hi-byte 8-bits, to prepare 
					  //  for the second byte.
      U0TXBUF = 0x00;                     // Send clocks to the ADC, this shifts
					  //  in the second 8 bits of data.
      complete();                         // Wait until all 8 bits have been received.           
      lo_byte = U0RXBUF;		  // Store this data in the lo-byte variable.
      
      temp = hi_byte + lo_byte;		  // Join the hi-byte and the lo-byte together.		
      CH0_data[CH0_buff] = temp >> 4;	  // Arrange the data format as 0xxx(h) - since
					  //  it's a 12-bit converter. 
      
      					  // ****** SCLKs for conversion ******
                           		  // > 28SCLKs are required for current conversion
      U0TXBUF = 0x00;			  //  This scheme gives 4 x 8SCLKs => 32SCLKs
      complete(); 
      trash = U0RXBUF;
      
      U0TXBUF = 0x00;                     
      complete();
      trash = U0RXBUF;
      
      U0TXBUF = 0x00;                     
      complete();
      trash = U0RXBUF;
      
      U0TXBUF = 0x00;                     
      complete();
      trash = U0RXBUF;
      
P3OUT |= CS;                              // De-assert CS HIGH
U0TXBUF = 0x00;                     	  // While CS* is de-asserted HIGH, must provide a high
complete();				  //  to low transition to release the data at the next access.
trash = U0RXBUF;			  // Clear the receiver
      }


P3OUT &= ~CS;                             // Assert CS LOW
      U0TXBUF = 0x00;                     // Send clocks to the ADC, this shifts 
					  //  in the first 8 bits of data.
      complete();                         // Wait until all 8 bits have been received.
      hi_byte = U0RXBUF;		  // Store this data in the hi-byte variable.
      hi_byte = hi_byte << 8;             // Left-shift the hi-byte 8-bits, to prepare 
					  //  for the second byte.
      U0TXBUF = 0x00;                     // Send clocks to the ADC, this shifts
					  //  in the second 8 bits of data.
      complete();                         // Wait until all 8 bits have been received.           
      lo_byte = U0RXBUF;		  // Store this data in the lo-byte variable.
      
      temp = hi_byte + lo_byte;		  // Join the hi-byte and the lo-byte together.		
      CH0_data[CH0_buff] = temp >> 4;	  // Arrange the data format as 0xxx(h) - since
					  //  it's a 12-bit converter. 
      
      					  // ****** SCLKs for conversion ******
                           		  // > 28SCLKs are required for current conversion
      U0TXBUF = 0x00;			  //  This scheme gives 4 x 8SCLKs => 32SCLKs
      complete(); 
      trash = U0RXBUF;
      
      U0TXBUF = 0x00;                     
      complete();
      trash = U0RXBUF;
      
      U0TXBUF = 0x00;                     
      complete();
      trash = U0RXBUF;
      
      U0TXBUF = 0x00;                     
      complete();
      trash = U0RXBUF;
      
      CH0_buff = CH0_buff + 1;		  // Increment CH0 buffer

P3OUT |= CS;                              // De-assert CS HIGH

U0TXBUF = 0x00;                     	  // While CS* is de-asserted HIGH, must provide a high
complete();				  //  to low transition to release the data at the next access.
trash = U0RXBUF;			  // Clear the receiver
      
P3OUT &= ~CS;                             // Assert CS LOW
      U0TXBUF = 0x00;                     // Send clocks to the ADC, this shifts 
					  //  in the first 8 bits of data.
      complete();                         // Wait until all 8 bits have been received.
      hi_byte = U0RXBUF;		  // Store this data in the hi-byte variable.
      hi_byte = hi_byte << 8;             // Left-shift the hi-byte 8-bits, to prepare 
					  //  for the second byte.
      U0TXBUF = 0x00;                     // Send clocks to the ADC, this shifts
					  //  in the second 8 bits of data.
      complete();                         // Wait until all 8 bits have been received.           
      lo_byte = U0RXBUF;		  // Store this data in the lo-byte variable.
      
      temp = hi_byte + lo_byte;		  // Join the hi-byte and the lo-byte together.		
      CH1_data[CH1_buff] = temp >> 4;	  // Arrange the data format as 0xxx(h) - since
					  //  it's a 12-bit converter. 
      
      					  // ****** SCLKs for conversion ******
                           		  // > 28SCLKs are required for current conversion
      U0TXBUF = 0x00;			  //  This scheme gives 4 x 8SCLKs => 32SCLKs
      complete(); 
      trash = U0RXBUF;
      
      U0TXBUF = 0x00;                     
      complete();
      trash = U0RXBUF;
      
      U0TXBUF = 0x00;                     
      complete();
      trash = U0RXBUF;
      
      U0TXBUF = 0x00;                     
      complete();
      trash = U0RXBUF;
      
      CH1_buff = CH1_buff + 1;		  // Increment CH1 buffer

P3OUT |= CS;                              // De-assert CS HIGH
U0TXBUF = 0x00;                     	  // While CS* is de-asserted HIGH, must provide a high
complete();				  //  to low transition to release the data at the next access.
trash = U0RXBUF;            		  // Clear the receiver
}
      

