#include <io.h>
;*****************************************************************************
;   MSP-FET430x110 Demo  - Software Interface with CD4511 7-Segment Decoder 
;
;   Description: Increment BCD value directly on P1.0-P1.3
;   ACLK = n/a, MCLK = SMCLK = default DCO ~ 800k 
;
;		 MSP430F1121     ^    CD4511
;             -----------------  |  ----------
;            |                 | |-|LT,BL     |  8
;            |             P1.0|-->|A      a-g|--\-> 7-Segment LED
;            |             P1.1|-->|B         |
;            |             P1.2|-->|C       LE|--|
;            |             P1.3|-->|D         |  V
;
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2001
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;*****************************************************************************
            .section    .text
RESET:      mov.w   #0x300,R1               ; Initialize 'x112x stackpointer
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop Watchdog Timer
SetupP2:    bic.b   #0x0F,&P1OUT            ; A,B,C,D = 0
            bis.b   #0x0F,&P1DIR            ; A,B,C,D = outputs
                                            ;
Mainloop:   push.b  &P1OUT                  ; Save current P1OUT
            setc                            ; 
            dadc.b  0(R1)                   ; Decimal increment P1OUT
            and.b   #0x0F,0(R1)             ; Lower nibble only
            bic.b   #0x0F,&P1OUT            ; Lower nibble only cleared
TX_4011:    bis.b   @R1+,&P1OUT             ; Incremented nibble
            call    #Delay                  ; Wait
            jmp     Mainloop                ; Repeat
                                            ; 
;----------------------------------------------------------------------------- 
Delay:      ;Software delay
;----------------------------------------------------------------------------- 
            push.w  #0xFFFF                 ; Delay to TOS
L1:         dec.w   0(R1)                   ; Decrement TOS
            jnz     L1                      ; Delay over?
            incd.w  R1                      ; Clean TOS
            ret                             ;
                                            ;
;----------------------------------------------------------------------------- 
;           Interrupt Vectors Used MSP430x11x1          
;----------------------------------------------------------------------------- 
            .section    .vectors
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
