#include <io.h>
;*****************************************************************************
;   MSP-FET430x110 Demo  - Software Parellel Interface with HC138 
;
;   Description: Transfer 3-bit binary value to HC138 using A, B, and C. 
;   Mainloop will increment 3-bits on P2.0 - P2.2 directly interfacing to
;    A-C inputs, rotating Y0 - Y7 outputs.
;   ACLK = n/a, MCLK = SMCLK = default DCO ~ 800k 
;
;		 MSP430F1121     ^     HC138
;             -----------------  |  ----------
;            |                 | |-|G1        |  8
;            |             P2.2|-->|C       Yx|--\->
;            |             P2.1|-->|B         |
;            |             P2.0|-->|A     /G2x|--|
;            |                 |   |          |  V
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0x300,R1               ; Initialize 'x112x stackpointer
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop Watchdog Timer
SetupP2:    bic.b   #0x07,&P2OUT             ; A,B,C = 0
            bis.b   #0x07,&P2DIR             ; A,B,C outputs
                                            ;
Mainloop:   push.b  &P2OUT                  ; P2OUT --> TOS
            inc.b   0(R1)                   ; Increment TOS
            and.b   #0x07,0(R1)             ; Expose three LSBs on TOS
            bic.b   #0x07,&P2OUT            ; P2.0 - 2 cleared
TX_HC138:   bis.b   @R1+,&P2OUT             ; P2.0 - 2 configured only 
            call    #Delay                  ; Wait
            jmp     Mainloop                ; Repeat
                                            ; 
;----------------------------------------------------------------------------- 
Delay:      ;Software delay
;----------------------------------------------------------------------------- 
            push.w  #0xFFFF                 ; Delay to TOS
L1:         dec.w   0(R1)                   ; Decrement TOS
            jnz     L1                      ; Delay over?
            incd.w  R1                      ; Clean-up Stack
            ret                             ;
                                            ;
;----------------------------------------------------------------------------- 
;           Interrupt Vectors Used MSP430x11x1          
;----------------------------------------------------------------------------- 
            .section    .vectors
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
