#include <io.h>
;******************************************************************************
;   MSP-FET430P120 Demo - BasicClock Configure RST/NMI as NMI
;
;   Deccription: Configure RST/NMI as NMI, hi/lo edge. Flash P1.0  inside of
;   NMI_ISR if NMI occurs. General enable interrupt in status register does
;   not need to be set for NMI. NMIIE does need to be reset, as NMI_ISR 
;   automatically clears NMI enable to prevent unintentional stack overflow
;   that could result from, bounch or uncontroller NMI's.
;   ACLK = n/a, MCLK = SMCLK = DCO ~ 800k
; 
;                 MSP430F123
;             -----------------
;         /|\|              XIN|-  
;          | |                 |
;          --|RST          XOUT|-
;            |                 |
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0x300,R1               ; Initialize stackpointer
SetupWDT:   mov.w   #WDTPW+WDTHOLD+WDTNMI+WDTNMIES,&WDTCTL  ; WDT off NMI hi/lo
            bis.b   #0x01,&P1DIR            ; P1.0 = output
            bic.b   #0x01,&P1OUT            ; P1.0 = reset
            bis.b   #NMIIE,&IE1             ; Enable NMI
                                            ;
Mainloop:   jmp     Mainloop                ; Endless Loop
                                            ;
;----------------------------------------------------------------------------- 
NMI_ISR:    ;Flash P1.0
;----------------------------------------------------------------------------- 
            bis.b   #0x01,&P1OUT            ; P1.0 = set
Wait:       push.w  #20000                  ; Delay value --> TOS
Wait1:      dec.w   0(R1)                   ; Decrement TOS
            jnz	    Wait1                   ; Delay over?
            incd.w  R1                      ; Clean up stack
            bic.b   #0x01,&P1OUT            ; P1.0 = reset
            bic.b   #NMIIFG,&IFG1           ; reclear NMI flag in case bounce
            bis.b   #NMIIE,&IE1             ; Re-enable NMI interrupt
            reti                            ; Done  
                                            ;
;----------------------------------------------------------------------------- 
;           Interrupt Vectors Used MSP430F12x         
;----------------------------------------------------------------------------- 
            .section    .vectors
            .org    0x1C,0xFF               ; NMI vector
            .word   NMI_ISR                 ; 
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
