#include <io.h>
;******************************************************************************
;   MSP-FET430P120 Demo - USART0 SPI Interface to TLC549 8-bit ADC
;
;   Description: This program demonstrate a USART0 in SPI mode interface to a 
;   TLC549 8-bit ADC. The TLC is read 16x, with the conversion code saved in
;   in RAM 0x200 - 0x21F. R15 used as pointer. 
;   ACLK = n/a, MCLK = SMCLK = default DCO ~ 800k, UCLK = DCO/2 
;   //** VCC must be at least 3v for TLC549 **// 
;
;                          MSP430F123(2)             
;                       -----------------
;                   /|\|              XIN|-  
;        TLC549      | |                 | 
;    -------------   --|RST          XOUT|-
;   |           CS|<---|P3.0             |
;   |      DATAOUT|--->|P3.2/SOMI        | 
; ~>| IN+  I/O CLK|<---|P3.3/UCLK0       |
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0x300,R1               ; Initialize stackpointer
            call    #Init_Sys               ; Initialize system
                                            ;
Mainloop:   mov.w   #0x1F,R15               ;
Read_549:   bic.b   #0x01,&P3OUT            ; Enable TLC549
            mov.b   #0x00,&TXBUF0           ; Dummy write to start SPI
L1:         bit.b   #URXIFG0,&IFG2          ; RXBUF ready?
            jnc      L1                     ; 1 = ready
            bis.b   #0x01,&P3OUT            ; Disable TLC549
                                            ;
            mov.b   &RXBUF0,0x200(R15)      ;
            dec.b   R15                     ;
            jc      Read_549                ;
            jmp     Mainloop                ; Repeat
                                            ;
;------------------------------------------------------------------------------ 
Init_Sys:   ;Subroutine to Initialize MSP430F123 Peripherials
;------------------------------------------------------------------------------ 
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop watchdog timer
SetupP3:    bis.b   #0x0C,&P3SEL            ; P3.2,3 SPI option select
            bis.b   #0x09,&P3DIR            ; P3.3,0 output direction
SetupSPI:   bis.b   #USPIE0,&ME2            ; Enable USART0 SPI
            bis.b   #CKPH+SSEL1+SSEL0+STC,&UTCTL0 ; SMCLK, 3-pin
            mov.b   #CHAR+SYNC+MM,&UCTL0    ; 8-bit SPI Master **SWRST**
            mov.b   #0x02,&UBR00            ; SMCLK/2 for baud rate
            clr.b   &UBR10                  ; SMCLK/2 for baud rate
            clr.b   &UMCTL0                 ; Clear modulation
            ret                             ; Return from subroutine
;
;------------------------------------------------------------------------------ 
;           Interrupt Vectors Used MSP430x12x         
;------------------------------------------------------------------------------ 
            .section    .vectors
            .org    0x1E,0xFF                ; POR, ext. Reset, Watchdog, Flash
            .word   RESET                    ; 
