#include <io.h>
;******************************************************************************
;   MSP-FET430P140 Demo - BasicClock MCLK configured with HF XTAL and OscFault
;
;   Description; Proper selection of an external HF XTAL for MCLK is shown with 
;   enabled OSC fault interrupt - if HF XTAL fails, NMI interrupt is requested.
;   HF XTAL can only source MCLK when OSC fault is clear. For demonstration
;   purposes P1.0 is set durring OSC fault. MCLK output on P5.4.   
;   ACLK = MCLK = LFXT1 = HFXTAL  
;   //** HF XTAL NOT INSTALLED ON FET **//
;
;                MSP430F149
;             -----------------
;         /|\|              XIN|-  
;          | |                 | HF XTAL (455k - 8Mhz)
;          --|RST          XOUT|-
;            |                 |
;            |             P1.0|-->LED
;            |             P5.4|-->MCLK = XTAL (or DCO durring fault)
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0xA00,R1               ; Initialize stackpointer
SetupWDT:   mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
            bis.b   #0x01,&P1DIR            ; P1.0 = output direction
            bic.b   #0x01,&P1OUT            ; P1.0 = reset
SetupP5:    bis.b   #0x10,&P5DIR            ; P5.4 output
            bis.b   #0x10,&P5SEL            ; P5.4 option
SetupBC:    bis.b   #XTS,&BCSCTL1           ; LFXT1 = HF XTAL
            bis.b   #OFIE,&IE1              ; Enable osc fault interrupt 
                                            ;           
Mainloop:   jmp     Mainloop                ; Repeat
                                            ;           
;------------------------------------------------------------------------------ 
NMI_ISR:    ;Only osc fault enabled, R15 used temporarly and not saved
;------------------------------------------------------------------------------ 
            bis.b   #0x01,&P1OUT            ; P1.0 = set
CheckOsc:   bic.b   #OFIFG,&IFG1            ; Clear OSC fault flag 
            mov.w   #0xFF,R15               ; R15 = Delay 
CheckOsc1:  dec.w   R15                     ; Additional delay to ensure start 
            jnz     CheckOsc1               ;
            bit.b   #OFIFG,&IFG1            ; OSC fault flag set? 
            jnz     CheckOsc                ; OSC Fault, clear flag again 
            bic.b   #0x01,&P1OUT            ; P1.0 = reset
            bis.b   #SELM1+SELM0,&BCSCTL2   ; MCLK = LFXT1 - now safe
            bis.b   #OFIE,&IE1              ; re-enable osc fault interrupt 
            reti                            ; return from interrupt
                                            ;
;------------------------------------------------------------------------------ 
;           Interrupt Vectors Used MSP430F14x         
;------------------------------------------------------------------------------ 
            .section    .vectors
            .org    0x1C,0xFF               ; NMI vector
            .word   NMI_ISR                 ; 
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ;
