#include <io.h>
;******************************************************************************
;   MSP-FET430P140 Demo - USART1 UART 19200 Echo ISR, HF XTAL ACLK 
;
;   Description; This program will echo a received character, RX ISR used. 
;   ACLK = MCLK = UCLK1 = = XT1 = 3.58MHz
;   Baud rate divider with 3.58Mhz XTAL @19200 = 3.58MHz/19200 = 186 (00BAh)
;   //*An external 3.58MHz XTAL on XIN XOUT is required for ACLK*//	  
;              
;                MSP430F149
;             -----------------
;         /|\|              XIN|-  
;          | |                 | 3.58MHz 
;          --|RST          XOUT|-
;            |                 |
;            |             P3.6|------------>
;            |                 | 19200 - 8N1 
;            |             P3.7|<------------
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0xA00,R1               ; Initialize 'F149 stackpointer
            call    #Init_Sys               ;
                                            ;   
Mainloop:   bis.b   #CPUOFF,R2              ; Enter LPM0
            nop                             ; Needed only for C-spy
                                            ;
;------------------------------------------------------------------------------ 
Init_Sys:   ;Initalize MSP430 system
;------------------------------------------------------------------------------ 
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
SetupBC:    bis.b   #XTS,&BCSCTL1           ; LFXT1 = HF XTAL
SetupOsc:   bic.b   #OFIFG,&IFG1            ; Clear OSC fault flag 
            mov.w   #0xFF,R15               ; R15 = Delay 
SetupOsc1:  dec.w   R15                     ; Additional delay to ensure start 
            jnz     SetupOsc1               ;
            bit.b   #OFIFG,&IFG1            ; OSC fault flag set? 
            jnz     SetupOsc                ; OSC Fault, clear flag again 
            bis.b   #SELM1+SELM0,&BCSCTL2   ; MCLK = LFXT1
SetupUART1: mov.b   #CHAR,&UCTL1            ; 8-bit characters
            mov.b   #SSEL0,&UTCTL1          ; UCLK1 = ACLK
            mov.b   #0xBA,&UBR01            ; 3.58Mhz 19200 - 186
            mov.b   #0x00,&UBR11            ; 3.58Mhz 19200
            clr.b   &UMCTL1                 ; 3.58MHz no modulation 19200
            bis.b   #UTXE1+URXE1,&ME2       ; Enable USART1 TXD/RXD
            bis.b   #URXIE1,&IE2            ; Enable USART1 RX interrupt
SetupP3:    bis.b   #0xC0,&P3SEL            ; P3.6,7 = USART1 option select
            bis.b   #0x40,&P3DIR            ; P3.6 = output direction 
            eint                            ; General enable interrupts
            ret                             ; Return from subroutine
                                            ;             
;------------------------------------------------------------------------------ 
USART1_ISR: ;Echo back RXed character, confirm TX buffer is ready first
;------------------------------------------------------------------------------ 
TX2:        bit.b   #UTXIFG1,&IFG2          ; USART1 TX buffer ready?
            jz      TX2                     ; Jump is TX buffer not ready
            mov.b   &RXBUF1,&TXBUF1         ; TX -> RXed character
            reti                            ; 
                                            ;
;------------------------------------------------------------------------------ 
;           Interrupt Vectors Used MSP430x13x/14x        
;------------------------------------------------------------------------------ 
            .section    .vectors
            .org    0x06,0xFF               ; 
            .word   USART1_ISR              ; USART1 receive
            .org    0x1E,0xFF               ; 
            .word   RESET                   ; POR, ext. Reset, Watchdog
