#include <io.h>
;******************************************************************************
;   MSP-FET430P140 Demo - WDT Toggle P1.0 Interval overflow ISR, DCO SMCLK
;
;   Description; Toggle P1.0 using software timed by the WDT ISR. Toggle rate 
;   is approximately 30ms based on default ~ 800khz DCO/SMCLK clock source 
;   used in this example for the WDT.  
;   ACLK = n/a, MCLK = SMCLK = default DCO ~ 800k
;
;		 MSP430F149
;             -----------------
;         /|\|              XIN|-  
;          | |                 |
;          --|RST          XOUT|-
;            |                 |
;            |             P1.0|-->LED
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0xA00,R1               ; Initialize 'F149 stackpointer
SetupWDT:   mov.w   #WDT_MDLY_32,&WDTCTL    ; WDT ~30ms interval timer 
            bis.b   #WDTIE,&IE1             ; Enable WDT interrupt 
SetupP1:    bis.b   #0x01,&P1DIR            ; P1.0 output
            eint                            ; Enable interrupts
                                            ;						  
Mainloop:   bis.w   #CPUOFF,R2              ; CPU off
            nop                             ; Required for C-spy
                                            ;
;------------------------------------------------------------------------------ 
WDT_ISR:    ;Toggle P1.0
;------------------------------------------------------------------------------ 
            xor.b   #0x01,&P1OUT            ; Toggle P1.0
            reti                            ;		 
                                            ;
;----------------------------------------------------------------------------- 
;           Interrupt Vectors Used MSP430x13x/14x        
;----------------------------------------------------------------------------- 
            .section    .vectors
            .org    0x14,0xFF               ; WDT Vector
            .word   WDT_ISR                 ;
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
