#include <io.h>
;*******************************************************************************
;   MSP-FET430P410 Demo - FLL+ clock, Runs internal DCO at 2.4576Mhz 
;
;   Description This program demostrates setting the internal DCO to run at
;   2.4576Mhz with auto-calibration by the FLL+.
;   ACLK = LFXT1 = 32768, MCLK = SMCLK = DCO = (n+1) x ACLK 
;   //*An external watch crystal is required on XIN/XOUT for ACLK*//	  
;
;                MSP430F413
;             -----------------
;         /|\|              XIN|-  
;          | |                 | 32khz xtal
;          --|RST          XOUT|-
;            |                 |
;            |             P1.1|--> MCLK = 2.4576Mhz
;            |                 |
;            |             P1.5|--> ACLK = 32khz
;            |                 |
;
;   B. Merritt
;   Texas Instruments Inc.
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;----------------------------------------------------------------------------- 
            .section    .text
RESET:      mov.w   #0x300,R1               ;initialize stackpointer
SetupWDT:   mov.w   #WDTPW+WDTHOLD,&WDTCTL  ;stop WDT
SetupFLL2:  bis.b   #FN_2,&SCFI0            ; x2 DCO, 4MHz nominal DCO  
            bis.b   #XCAP18PF,&FLL_CTL0     ;set load capacitance for xtal
            mov.b   #74,&SCFQCTL            ;(74+1) x 32768 = 2.4576Mhz
                                            ;
SetupPorts: bis.b   #0x22,&P1DIR            ;P1.1 & P1.5 to output direction
            bis.b   #0x22,&P1SEL            ;P1.1 & P1.5 to output MCLK & ACLK
                                            ;
Mainloop:   jmp     Mainloop                ;loop with CPU running
                                            ;
;----------------------------------------------------------------------------- 
;           MSP430F41x interrupt vectors used         
;----------------------------------------------------------------------------- 
            .section    .vectors
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
