#include <io.h>
;******************************************************************************
;   MSP-FET430P440 Demo - Toggle P5.1 using ISR, DCO SMCLK 
;
;   Description; This program will toggle P5.1 by xor'ing P5.1 inside of
;   a basic timer ISR. Default DCO used to clock basic timer. 
;   ACLK = LFXT1 = 32768, MCLK = SMCLK = default DCO ~ 800k
;
;                MSP430F449
;             -----------------
;         /|\|              XIN|-  
;          | |                 |
;          --|RST          XOUT|-
;            |                 |
;            |             P5.1|-->LED
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0xA00,R1               ; Initialize '449 stackpointer
Init_Sys:   mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
SetupBT:    mov.b   #BTSSEL+BTIP2+BTIP1+BTIP0,&BTCTL   ; ~244us Int. 
            bis.b   #BTIE,&IE2              ; Enable Basic Timer interrupt        
SetupP5:    bis.b   #0x02,&P5DIR            ; P5.1 output
            eint                            ; Enable interrupts
                                            ;						  
Mainloop:   bis.w   #CPUOFF,R2              ; CPU off
            nop                             ; Required for C-spy
                                            ;
;------------------------------------------------------------------------------
BT_ISR:     ;Toggle P5.1
;------------------------------------------------------------------------------
            xor.b   #0x02,&P5OUT            ; Toggle P5.1
            reti                            ;		 
                                            ;   
;----------------------------------------------------------------------------- 
;           Interrupt Vectors Used MSP430F44x         
;----------------------------------------------------------------------------- 
            .section    .vectors
            .org    0x00,0xFF               ; BT Vector
            .word   BT_ISR                  ; 
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
